@@ -15,11 +15,15 @@
 #   
 #   You should have received a copy of the GNU General Public License
 #   along with this program; if not, write to the Free Software
-#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+#   Foundation, Inc.,  51 Franklin Street, Fifth Floor, Boston, MA
+#   02110-1301 USA
 #
 ###################################################################
 
-evcent <- function(graph, v=igraph.vs.all(graph)) {
+evcent <- function(graph, v=V(graph)) {
+  if (!is.igraph(graph)) {
+    stop("Not a graph object")
+  }
 
   ad <- get.adjacency(graph)
 
