@@ -828,6 +828,8 @@ int igraph_layout_lgl(const igraph_t *graph, igraph_matrix_t *res,
 		      igraph_real_t repulserad, igraph_real_t cellsize, igraph_integer_t root);
 int igraph_layout_reingold_tilford(const igraph_t *graph, igraph_matrix_t *res,
               long int root);
+int igraph_layout_reingold_tilford_circular(const igraph_t *graph,
+					    igraph_matrix_t *res, long int root);
 
 int igraph_layout_random_3d(const igraph_t *graph, igraph_matrix_t *res);
 int igraph_layout_sphere(const igraph_t *graph, igraph_matrix_t *res);
@@ -1337,6 +1339,7 @@ int igraph_revolver_d(const igraph_t *graph,
 		     igraph_vector_t *expected,
 		     igraph_real_t *logprob,
 		     igraph_real_t *lognull,
+		     igraph_real_t *logmax,
 		     const igraph_vector_t *debug,
 		     igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_d(const igraph_t *graph,
@@ -1346,6 +1349,7 @@ int igraph_revolver_mes_d(const igraph_t *graph,
 			 igraph_vector_t *cites,
 			 const igraph_vector_t *debug,
 			 igraph_vector_ptr_t *debugres,
+			 igraph_real_t *logmax,
 			 const igraph_vector_t *st,
 			 igraph_integer_t pmaxind);
 int igraph_revolver_st_d(const igraph_t *graph,
@@ -1362,6 +1366,10 @@ int igraph_revolver_error_d(const igraph_t *graph,
 			   igraph_integer_t maxind,
 			   igraph_real_t *logprob,
 			   igraph_real_t *lognull);
+int igraph_revolver_error2_d(const igraph_t *graph,
+			     const igraph_vector_t *kernel,
+			     igraph_real_t *logprob,
+			     igraph_real_t *lognull);
 
 int igraph_revolver_ad(const igraph_t *graph,
 		      igraph_integer_t niter,
@@ -1373,6 +1381,7 @@ int igraph_revolver_ad(const igraph_t *graph,
 		      igraph_matrix_t *expected,
 		      igraph_real_t *logprob,
 		      igraph_real_t *lognull,
+		      igraph_real_t *logmax,
 		      const igraph_matrix_t *debug,
 		      igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_ad(const igraph_t *graph,
@@ -1382,6 +1391,7 @@ int igraph_revolver_mes_ad(const igraph_t *graph,
 			  igraph_matrix_t *cites,
 			  const igraph_matrix_t *debug,
 			  igraph_vector_ptr_t *debugres,
+			  igraph_real_t *logmax,
 			  const igraph_vector_t *st,
 			  igraph_integer_t pmaxind,
 			  igraph_integer_t agebins);
@@ -1401,6 +1411,10 @@ int igraph_revolver_error_ad(const igraph_t *graph,
 			    igraph_integer_t pagebins,
 			    igraph_real_t *logprob,
 			    igraph_real_t *lognull);
+int igraph_revolver_error2_ad(const igraph_t *graph,
+			      const igraph_matrix_t *kernel,
+			      igraph_real_t *logprob,
+			      igraph_real_t *lognull);
 
 int igraph_revolver_ade(const igraph_t *graph,
 		       igraph_integer_t niter,
@@ -1413,6 +1427,7 @@ int igraph_revolver_ade(const igraph_t *graph,
 		       igraph_array3_t *expected,
 		       igraph_real_t *logprob,
 		       igraph_real_t *lognull,
+		       igraph_real_t *logmax,
 		       const igraph_matrix_t *debug,
 		       igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_ade(const igraph_t *graph, 
@@ -1422,6 +1437,7 @@ int igraph_revolver_mes_ade(const igraph_t *graph,
 			   igraph_array3_t *cites,
 			   const igraph_matrix_t *debug,
 			   igraph_vector_ptr_t *debugres,
+			   igraph_real_t *logmax,
 			   const igraph_vector_t *st,
 			   const igraph_vector_t *cats,
 			   igraph_integer_t pnocats,
@@ -1448,6 +1464,11 @@ int igraph_revolver_error_ade(const igraph_t *graph,
 			     igraph_integer_t pagebins,
 			     igraph_real_t *logprob,
 			     igraph_real_t *lognull);
+int igraph_revolver_error2_ade(const igraph_t *graph,
+			       const igraph_array3_t *kernel,
+			       const igraph_vector_t *cats,
+			       igraph_real_t *logprob,
+			       igraph_real_t *lognull);
 
 int igraph_revolver_e(const igraph_t *graph,
 		     igraph_integer_t niter,
@@ -1459,6 +1480,7 @@ int igraph_revolver_e(const igraph_t *graph,
 		     igraph_vector_t *expected,
 		     igraph_real_t *logprob,
 		     igraph_real_t *lognull,
+		     igraph_real_t *logmax,
 		     const igraph_vector_t *debug,
 		     igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_e(const igraph_t *graph,
@@ -1468,6 +1490,7 @@ int igraph_revolver_mes_e(const igraph_t *graph,
 			 igraph_vector_t *cites,
 			 const igraph_vector_t *debug,
 			 igraph_vector_ptr_t *debugres,
+			 igraph_real_t *logmax,
 			 const igraph_vector_t *st,
 			 const igraph_vector_t *cats,
 			 igraph_integer_t pnocats);
@@ -1488,6 +1511,11 @@ int igraph_revolver_error_e(const igraph_t *graph,
 			   igraph_integer_t pnocats,
 			   igraph_real_t *logprob,
 			   igraph_real_t *lognull);
+int igraph_revolver_error2_e(const igraph_t *graph,
+			     const igraph_vector_t *kernel,
+			     const igraph_vector_t *cats,
+			     igraph_real_t *logprob,
+			     igraph_real_t *lognull);
 
 int igraph_revolver_de(const igraph_t *graph,
 		      igraph_integer_t niter,
@@ -1499,6 +1527,7 @@ int igraph_revolver_de(const igraph_t *graph,
 		      igraph_matrix_t *expected,
 		      igraph_real_t *logprob,
 		      igraph_real_t *lognull,
+		      igraph_real_t *logmax,
 		      const igraph_matrix_t *debug,
 		      igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_de(const igraph_t *graph,
@@ -1508,6 +1537,7 @@ int igraph_revolver_mes_de(const igraph_t *graph,
 			  igraph_matrix_t *cites,
 			  const igraph_matrix_t *debug,
 			  igraph_vector_ptr_t *debugres,
+			  igraph_real_t *logmax,
 			  const igraph_vector_t *st,
 			  const igraph_vector_t *cats,
 			  igraph_integer_t pnocats,
@@ -1531,6 +1561,11 @@ int igraph_revolver_error_de(const igraph_t *graph,
 			    igraph_integer_t pmaxind,
 			    igraph_real_t *logprob,
 			    igraph_real_t *lognull);
+int igraph_revolver_error2_de(const igraph_t *graph,
+			      const igraph_matrix_t *kernel,
+			      const igraph_vector_t *cats,
+			      igraph_real_t *logprob,
+			      igraph_real_t *lognull);
 
 int igraph_revolver_l(const igraph_t *graph,
 		     igraph_integer_t niter,
@@ -1542,6 +1577,7 @@ int igraph_revolver_l(const igraph_t *graph,
 		     igraph_vector_t *expected,
 		     igraph_real_t *logprob,
 		     igraph_real_t *lognull,
+		     igraph_real_t *logmax,
 		     const igraph_vector_t *debug,
 		     igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_l(const igraph_t *graph,
@@ -1551,6 +1587,7 @@ int igraph_revolver_mes_l(const igraph_t *graph,
 			 igraph_vector_t *cites,
 			 const igraph_vector_t *debug,
 			 igraph_vector_ptr_t *debugres,
+			 igraph_real_t *logmax,
 			 const igraph_vector_t *st,
 			 igraph_integer_t pagebins);
 int igraph_revolver_st_l(const igraph_t *graph,
@@ -1567,6 +1604,10 @@ int igraph_revolver_error_l(const igraph_t *graph,
 			   igraph_integer_t pagebins,
 			   igraph_real_t *logprob,
 			   igraph_real_t *lognull);
+int igraph_revolver_error2_l(const igraph_t *graph,
+			     const igraph_vector_t *kernel,			     
+			     igraph_real_t *logprob,
+			     igraph_real_t *lognull);
 
 int igraph_revolver_dl(const igraph_t *graph,
 		      igraph_integer_t niter,
@@ -1578,6 +1619,7 @@ int igraph_revolver_dl(const igraph_t *graph,
 		      igraph_matrix_t *expected,
 		      igraph_real_t *logprob,
 		      igraph_real_t *lognull,
+		      igraph_real_t *logmax,
 		      const igraph_matrix_t *debug,
 		      igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_dl(const igraph_t *graph,
@@ -1587,6 +1629,7 @@ int igraph_revolver_mes_dl(const igraph_t *graph,
 			  igraph_matrix_t *cites,
 			  const igraph_matrix_t *debug,
 			  igraph_vector_ptr_t *debugres,
+			  igraph_real_t *logmax,
 			  const igraph_vector_t *st,
 			  igraph_integer_t pmaxind,
 			  igraph_integer_t pagebins);
@@ -1606,6 +1649,10 @@ int igraph_revolver_error_dl(const igraph_t *graph,
 			    igraph_integer_t pmaxind,
 			    igraph_real_t *logprob,
 			    igraph_real_t *lognull);
+int igraph_revolver_error2_dl(const igraph_t *graph,
+			      const igraph_matrix_t *kernel,
+			      igraph_real_t *logprob,
+			      igraph_real_t *lognull);
 
 int igraph_revolver_el(const igraph_t *graph,
 		      igraph_integer_t niter,
@@ -1618,6 +1665,7 @@ int igraph_revolver_el(const igraph_t *graph,
 		      igraph_matrix_t *expected,
 		      igraph_real_t *logprob,
 		      igraph_real_t *lognull,
+		      igraph_real_t *logmax,
 		      const igraph_matrix_t *debug,
 		      igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_el(const igraph_t *graph,
@@ -1627,6 +1675,7 @@ int igraph_revolver_mes_el(const igraph_t *graph,
 			  igraph_matrix_t *cites,
 			  const igraph_matrix_t *debug,
 			  igraph_vector_ptr_t *debugres,
+			  igraph_real_t *logmax,
 			  const igraph_vector_t *st,
 			  const igraph_vector_t *cats,
 			  igraph_integer_t pnocats,
@@ -1650,6 +1699,11 @@ int igraph_revolver_error_el(const igraph_t *graph,
 			    igraph_integer_t pagebins,
 			    igraph_real_t *logprob,
 			    igraph_real_t *lognull);
+int igraph_revolver_error2_el(const igraph_t *graph,
+			      const igraph_matrix_t *kernel,
+			      const igraph_vector_t *cats,
+			      igraph_real_t *logprob,
+			      igraph_real_t *lognull);
 
 int igraph_revolver_r(const igraph_t *graph,
 		     igraph_integer_t niter,
@@ -1661,6 +1715,7 @@ int igraph_revolver_r(const igraph_t *graph,
 		     igraph_vector_t *expected,
 		     igraph_real_t *logprob,
 		     igraph_real_t *lognull,
+		     igraph_real_t *logmax,
 		     const igraph_vector_t *debug,
 		     igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_r(const igraph_t *graph,
@@ -1670,6 +1725,7 @@ int igraph_revolver_mes_r(const igraph_t *graph,
 			 igraph_vector_t *cites,
 			 const igraph_vector_t *debug,
 			 igraph_vector_ptr_t *debugres,
+			 igraph_real_t *logmax,
 			 const igraph_vector_t *st,
 			 igraph_integer_t window,
 			 igraph_integer_t maxind);
@@ -1690,6 +1746,11 @@ int igraph_revolver_error_r(const igraph_t *graph,
 			   igraph_integer_t maxind,			   
 			   igraph_real_t *logprob,
 			   igraph_real_t *lognull);
+int igraph_revolver_error2_r(const igraph_t *graph,
+			     const igraph_vector_t *kernel,
+			     igraph_integer_t window,
+			     igraph_real_t *logprob,
+			     igraph_real_t *lognull);
 
 int igraph_revolver_ar(const igraph_t *graph,
 		      igraph_integer_t niter,
@@ -1702,6 +1763,7 @@ int igraph_revolver_ar(const igraph_t *graph,
 		      igraph_matrix_t *expected,
 		      igraph_real_t *logprob,
 		      igraph_real_t *lognull,
+		      igraph_real_t *logmax,
 		      const igraph_matrix_t *debug,
 		      igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_ar(const igraph_t *graph,
@@ -1711,6 +1773,7 @@ int igraph_revolver_mes_ar(const igraph_t *graph,
 			  igraph_matrix_t *cites,
 			  const igraph_matrix_t *debug,
 			  igraph_vector_ptr_t *debugres,
+			  igraph_real_t *logmax,
 			  const igraph_vector_t *st,
 			  igraph_integer_t pagebins,
 			  igraph_integer_t pwindow,
@@ -1734,6 +1797,11 @@ int igraph_revolver_error_ar(const igraph_t *graph,
 			    igraph_integer_t maxind,			   
 			    igraph_real_t *logprob,
 			    igraph_real_t *lognull);
+int igraph_revolver_error2_ar(const igraph_t *graph, 
+			      const igraph_matrix_t *kernel,
+			      igraph_integer_t window, 
+			      igraph_real_t *logprob, 
+			      igraph_real_t *lognull);
 
 int igraph_revolver_di(const igraph_t *graph,
 		      igraph_integer_t niter,
@@ -1745,6 +1813,7 @@ int igraph_revolver_di(const igraph_t *graph,
 		      igraph_matrix_t *expected,
 		      igraph_real_t *logprob,
 		      igraph_real_t *lognull,
+		      igraph_real_t *logmax,
 		      const igraph_matrix_t *debug,
 		      igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_di(const igraph_t *graph,
@@ -1754,6 +1823,7 @@ int igraph_revolver_mes_di(const igraph_t *graph,
 			  igraph_matrix_t *cites,
 			  const igraph_matrix_t *debug,
 			  igraph_vector_ptr_t *debugres,
+			  igraph_real_t *logmax,
 			  const igraph_vector_t *st,
 			  const igraph_vector_t *cats,
 			  igraph_integer_t pnocats,
@@ -1777,6 +1847,11 @@ int igraph_revolver_error_di(const igraph_t *graph,
 			    igraph_integer_t pmaxind,
 			    igraph_real_t *logprob,
 			    igraph_real_t *lognull);
+int igraph_revolver_error2_di(const igraph_t *graph,
+			      const igraph_matrix_t *kernel,
+			      const igraph_vector_t *cats,
+			      igraph_real_t *logprob,
+			      igraph_real_t *lognull);
 
 int igraph_revolver_adi(const igraph_t *graph,
 		       igraph_integer_t niter,
@@ -1789,6 +1864,7 @@ int igraph_revolver_adi(const igraph_t *graph,
 		       igraph_array3_t *expected,
 		       igraph_real_t *logprob,
 		       igraph_real_t *lognull,
+		       igraph_real_t *logmax,
 		       const igraph_matrix_t *debug,
 		       igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_adi(const igraph_t *graph,
@@ -1798,6 +1874,7 @@ int igraph_revolver_mes_adi(const igraph_t *graph,
 			   igraph_array3_t *cites,
 			   const igraph_matrix_t *debug,
 			   igraph_vector_ptr_t *debugres,
+			   igraph_real_t *logmax,
 			   const igraph_vector_t *st,
 			   const igraph_vector_t *cats,
 			   igraph_integer_t pnocats,
@@ -1824,6 +1901,12 @@ int igraph_revolver_error_adi(const igraph_t *graph,
 			     igraph_integer_t pagebins,
 			     igraph_real_t *logprob,
 			     igraph_real_t *lognull);
+int igraph_revolver_error2_adi(const igraph_t *graph,
+			       const igraph_array3_t *kernel,
+			       const igraph_vector_t *cats,
+			       igraph_real_t *logprob,
+			       igraph_real_t *lognull);
+
 int igraph_revolver_il(const igraph_t *graph,
 		      igraph_integer_t niter,
 		      igraph_integer_t agebins,
@@ -1835,6 +1918,7 @@ int igraph_revolver_il(const igraph_t *graph,
 		      igraph_matrix_t *expected,
 		      igraph_real_t *logprob,
 		      igraph_real_t *lognull,
+		      igraph_real_t *logmax,
 		      const igraph_matrix_t *debug,
 		      igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_il(const igraph_t *graph,
@@ -1844,6 +1928,7 @@ int igraph_revolver_mes_il(const igraph_t *graph,
 			  igraph_matrix_t *cites,
 			  const igraph_matrix_t *debug,
 			  igraph_vector_ptr_t *debugres,
+			  igraph_real_t *logmax,
 			  const igraph_vector_t *st,
 			  const igraph_vector_t *cats,
 			  igraph_integer_t pnocats,
@@ -1867,6 +1952,12 @@ int igraph_revolver_error_il(const igraph_t *graph,
 			    igraph_integer_t pagebins,
 			    igraph_real_t *logprob,
 			    igraph_real_t *lognull);
+int igraph_revolver_error2_il(const igraph_t *graph,
+			      const igraph_matrix_t *kernel,
+			      const igraph_vector_t *cats,
+			      igraph_real_t *logprob,
+			      igraph_real_t *lognull);
+
 int igraph_revolver_ir(const igraph_t *graph,
 		      igraph_integer_t niter,
 		      igraph_integer_t window,
@@ -1878,6 +1969,7 @@ int igraph_revolver_ir(const igraph_t *graph,
 		      igraph_matrix_t *expected,
 		      igraph_real_t *logprob,
 		      igraph_real_t *lognull,
+		      igraph_real_t *logmax,
 		      const igraph_matrix_t *debug,
 		      igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_ir(const igraph_t *graph,
@@ -1887,6 +1979,7 @@ int igraph_revolver_mes_ir(const igraph_t *graph,
 			  igraph_matrix_t *cites,
 			  const igraph_matrix_t *debug,
 			  igraph_vector_ptr_t *debugres,
+			  igraph_real_t *logmax,
 			  const igraph_vector_t *st,
 			  igraph_integer_t pwindow,
 			  const igraph_vector_t *cats,
@@ -1914,6 +2007,13 @@ int igraph_revolver_error_ir(const igraph_t *graph,
 			    igraph_integer_t pmaxind,
 			    igraph_real_t *logprob,
 			    igraph_real_t *lognull);
+int igraph_revolver_error2_ir(const igraph_t *graph,
+			      const igraph_matrix_t *kernel,
+			      const igraph_vector_t *cats,
+			      igraph_integer_t window,
+			      igraph_real_t *logprob,
+			      igraph_real_t *lognull);
+
 int igraph_revolver_air(const igraph_t *graph,
 		       igraph_integer_t niter,
 		       igraph_integer_t window,
@@ -1926,6 +2026,7 @@ int igraph_revolver_air(const igraph_t *graph,
 		       igraph_array3_t *expected,
 		       igraph_real_t *logprob,
 		       igraph_real_t *lognull,
+		       igraph_real_t *logmax,
 		       const igraph_matrix_t *debug,
 		       igraph_vector_ptr_t *debugres);
 int igraph_revolver_mes_air(const igraph_t *graph,
@@ -1935,6 +2036,7 @@ int igraph_revolver_mes_air(const igraph_t *graph,
 			   igraph_array3_t *cites,
 			   const igraph_matrix_t *debug,
 			   igraph_vector_ptr_t *debugres,
+			   igraph_real_t *logmax,
 			   const igraph_vector_t *st,
 			   igraph_integer_t pwindow,
 			   const igraph_vector_t *cats,
@@ -1965,6 +2067,12 @@ int igraph_revolver_error_air(const igraph_t *graph,
 			     igraph_integer_t pagebins,
 			     igraph_real_t *logprob,
 			     igraph_real_t *lognull);
+int igraph_revolver_error2_air(const igraph_t *graph,
+			       const igraph_array3_t *kernel,
+			       const igraph_vector_t *cats,
+			       igraph_integer_t window,
+			       igraph_real_t *logprob,
+			       igraph_real_t *lognull);
 
 /* Should be moved to to types.h? */
 typedef struct igraph_i_lazy_adjedgelist_t {
